from code42cli.arguments import add_profile_name_arg
from code42cli.profile.profile import get_profile
from code42cli.securitydata.cursor_store import FileEventCursorStore


def init(subcommand_parser):
    """Sets up the `clear-checkpoint` subcommand for cleared the stored checkpoint for `incremental` mode.
        Args:
            subcommand_parser: The subparsers group created by the parent parser.
    """
    parser = subcommand_parser.add_parser("clear-checkpoint")
    add_profile_name_arg(parser)
    parser.set_defaults(func=clear_checkpoint)


def clear_checkpoint(args):
    """Removes the stored checkpoint that keeps track of the last event you got.
        To use, run `code42 clear-checkpoint`.
        This affects `incremental` mode by causing it to behave like it has never been run before.
    """
    profile_name = args.profile_name or get_profile().name
    FileEventCursorStore(profile_name).replace_stored_insertion_timestamp(None)



import code42cli.arguments as main_args
from code42cli.securitydata.arguments import main as securitydata_main_args
from code42cli.securitydata.arguments import search as search_args
from code42cli.securitydata.extraction import extract
from code42cli.securitydata.logger_factory import get_logger_for_server
from code42cli.securitydata.options import ServerProtocol


def init(subcommand_parser):
    """Sets up the `send-to` subcommand for sending logs to a server, such as SysLog.
            Use `-h` after any subcommand for usage.
        Args:
            subcommand_parser: The subparsers group created by the parent parser
    """
    parser = subcommand_parser.add_parser("send-to")
    parser.set_defaults(func=send_to)
    _add_server_arg(parser)
    _add_protocol_arg(parser)
    search_args.add_arguments_to_parser(parser)
    securitydata_main_args.add_arguments_to_parser(parser)
    main_args.add_arguments_to_parser(parser)


def send_to(args):
    """Activates 'send-to' command. It gets security events and logs them to the given server."""
    logger = get_logger_for_server(args.server, args.protocol, args.format)
    extract(logger, args)


def _add_server_arg(parser):
    parser.add_argument(action="store", dest="server", help="The server address to send output to.")


def _add_protocol_arg(parser):
    parser.add_argument(
        "-p",
        "--protocol",
        action="store",
        dest="protocol",
        choices=ServerProtocol(),
        default=ServerProtocol.UDP,
        help="Protocol used to send logs to server.",
    )



from py42 import debug_level
from py42 import settings
from py42.sdk import SDK

from code42cli.util import print_error


def create_sdk(profile, is_debug_mode):
    if is_debug_mode:
        settings.debug_level = debug_level.DEBUG
    try:
        password = profile.get_password()
        return SDK.create_using_local_account(profile.authority_url, profile.username, password)
    except Exception:
        print_error(
            u"Invalid credentials or host address. "
            u"Verify your profile is set up correctly and that you are supplying the correct password."
        )
        exit(1)


def validate_connection(authority_url, username, password):
    try:
        SDK.create_using_local_account(authority_url, username, password)
        return True
    except:
        print(username, password, authority_url)
        return False



import code42cli.arguments as main_args
from code42cli.securitydata.arguments import main as securitydata_main_args
from code42cli.securitydata.arguments import search as search_args
from code42cli.securitydata.extraction import extract
from code42cli.securitydata.logger_factory import get_logger_for_file


def init(subcommand_parser):
    """Sets up the `write-to` subcommand for writing logs to a file.
            Use `-h` after any subcommand for usage.
        Args:
            subcommand_parser: The subparsers group created by the parent parser.
    """
    parser = subcommand_parser.add_parser("write-to")
    parser.set_defaults(func=write_to)
    _add_filename_subcommand(parser)
    search_args.add_arguments_to_parser(parser)
    securitydata_main_args.add_arguments_to_parser(parser)
    main_args.add_arguments_to_parser(parser)


def write_to(args):
    """Activates 'write-to' command. It gets security events and writes them to the given file."""
    logger = get_logger_for_file(args.output_file, args.format)
    extract(logger, args)


def _add_filename_subcommand(parser):
    parser.add_argument(
        action=u"store", dest=u"output_file", help=u"The name of the local file to send output to."
    )



from __future__ import print_function

from getpass import getpass

import keyring

from code42cli.profile.config import get_config_accessor, ConfigAccessor

_ROOT_SERVICE_NAME = u"code42cli"


def get_stored_password(profile_name):
    """Gets your currently stored password for the given profile name."""
    profile = _get_profile(profile_name)
    service_name = _get_service_name(profile.name)
    username = _get_username(profile)
    password = keyring.get_password(service_name, username)
    return password


def get_password_from_prompt():
    """Prompts you and returns what you input."""
    return getpass()


def set_password(profile_name, new_password):
    """Sets your password for the given profile name."""
    profile = _get_profile(profile_name)
    service_name = _get_service_name(profile.name)
    username = _get_username(profile)
    keyring.set_password(service_name, username, new_password)
    print(u"'Code42 Password' updated.")


def _get_profile(profile_name):
    accessor = get_config_accessor()
    return accessor.get_profile(profile_name)


def _get_service_name(profile_name):
    return u"{}::{}".format(_ROOT_SERVICE_NAME, profile_name)


def _get_username(profile):
    return profile[ConfigAccessor.USERNAME_KEY]



class OutputFormat(object):
    CEF = "CEF"
    JSON = "JSON"
    RAW = "RAW-JSON"

    def __iter__(self):
        return iter([self.CEF, self.JSON, self.RAW])


class ExposureType(object):
    SHARED_VIA_LINK = "SharedViaLink"
    SHARED_TO_DOMAIN = "SharedToDomain"
    APPLICATION_READ = "ApplicationRead"
    CLOUD_STORAGE = "CloudStorage"
    REMOVABLE_MEDIA = "RemovableMedia"
    IS_PUBLIC = "IsPublic"

    def __iter__(self):
        return iter(self._as_list())

    def __len__(self):
        return len(self._as_list())

    def _as_list(self):
        return [
            self.SHARED_VIA_LINK,
            self.SHARED_TO_DOMAIN,
            self.APPLICATION_READ,
            self.CLOUD_STORAGE,
            self.REMOVABLE_MEDIA,
            self.IS_PUBLIC,
        ]


class ServerProtocol(object):
    TCP = "TCP"
    UDP = "UDP"

    def __iter__(self):
        return iter([self.TCP, self.UDP])



from __future__ import print_function

import code42cli.arguments as main_args
import code42cli.profile.password as password
from code42cli.compat import str
from code42cli.profile.config import get_config_accessor, ConfigAccessor
from code42cli.util import (
    get_input,
    print_error,
    print_set_profile_help,
    print_no_existing_profile_message,
)
from code42cli.sdk_client import validate_connection


class Code42Profile(object):
    def __init__(self, profile):
        self._profile = profile

    @property
    def name(self):
        return self._profile.name

    @property
    def authority_url(self):
        return self._profile[ConfigAccessor.AUTHORITY_KEY]

    @property
    def username(self):
        return self._profile[ConfigAccessor.USERNAME_KEY]

    @property
    def ignore_ssl_error(self):
        return self._profile[ConfigAccessor.IGNORE_SSL_ERRORS_KEY]

    def get_password(self):
        pwd = password.get_stored_password(self.name)
        if not pwd:
            pwd = password.get_password_from_prompt()
        return pwd

    def __str__(self):
        return u"{0}: Username={1}, Authority URL={2}".format(
            self.name, self.username, self.authority_url
        )


def init(subcommand_parser):
    """Sets up the `profile` subcommand with `show` and `set` subcommands.
            `show` will print the current profile while `set` will modify profile properties.
            Use `-h` after any subcommand for usage.
        Args:
            subcommand_parser: The subparsers group created by the parent parser.
    """
    parser_profile = subcommand_parser.add_parser(u"profile")
    profile_subparsers = parser_profile.add_subparsers()

    parser_for_show = profile_subparsers.add_parser(u"show")
    parser_for_set = profile_subparsers.add_parser(u"set")
    parser_for_reset_password = profile_subparsers.add_parser(u"reset-pw")
    parser_for_list = profile_subparsers.add_parser(u"list")
    parser_for_use = profile_subparsers.add_parser(u"use")

    parser_for_show.set_defaults(func=show_profile)
    parser_for_set.set_defaults(func=set_profile)
    parser_for_reset_password.set_defaults(func=prompt_for_password_reset)
    parser_for_list.set_defaults(func=list_profiles)
    parser_for_use.set_defaults(func=use_profile)

    main_args.add_profile_name_arg(parser_for_show)
    main_args.add_profile_name_arg(parser_for_reset_password)
    _add_args_to_set_command(parser_for_set)
    _add_positional_profile_arg(parser_for_use)


def get_profile(profile_name=None):
    """Returns the profile for the given name."""
    accessor = get_config_accessor()
    try:
        profile = accessor.get_profile(profile_name)
        return Code42Profile(profile)
    except Exception as ex:
        print_error(str(ex))
        print_set_profile_help()
        exit(1)


def show_profile(args):
    """Prints the given profile to stdout."""
    profile = get_profile(args.profile_name)
    print(u"\n{0}:".format(profile.name))
    print(u"\t* {0} = {1}".format(ConfigAccessor.USERNAME_KEY, profile.username))
    print(u"\t* {0} = {1}".format(ConfigAccessor.AUTHORITY_KEY, profile.authority_url))
    print(u"\t* {0} = {1}".format(ConfigAccessor.IGNORE_SSL_ERRORS_KEY, profile.ignore_ssl_error))
    if password.get_stored_password(profile.name) is not None:
        print(u"\t* A password is set.")
    print(u"")


def set_profile(args):
    """Sets the given profile using command line arguments."""
    _verify_args_for_set(args)
    accessor = get_config_accessor()
    accessor.create_profile_if_not_exists(args.profile_name)
    _try_set_authority_url(args, accessor)
    _try_set_username(args, accessor)
    _try_set_ignore_ssl_errors(args, accessor)
    _prompt_for_allow_password_set(args)


def prompt_for_password_reset(args):
    """Securely prompts for your password and then stores it using keyring."""
    profile = get_profile(args.profile_name)
    new_password = password.get_password_from_prompt()
    if not validate_connection(profile.authority_url, profile.username, new_password):
        print_error(
            "Your password was not saved because your credentials failed to validate. "
            "Check your network connection and the spelling of your username and server URL."
        )
        exit(1)
    password.set_password(profile.name, new_password)


def list_profiles(*args):
    """Lists all profiles that exist for this OS user."""
    accessor = get_config_accessor()
    profiles = accessor.get_all_profiles()
    if not profiles:
        print_no_existing_profile_message()
        return
    for profile in profiles:
        profile = Code42Profile(profile)
        print(profile)


def use_profile(args):
    """Changes the default profile to the given one."""
    accessor = get_config_accessor()
    try:
        accessor.switch_default_profile(args.profile_name)
    except Exception as ex:
        print_error(str(ex))
        exit(1)


def _add_args_to_set_command(parser_for_set):
    main_args.add_profile_name_arg(parser_for_set)
    _add_authority_arg(parser_for_set)
    _add_username_arg(parser_for_set)
    _add_disable_ssl_errors_arg(parser_for_set)
    _add_enable_ssl_errors_arg(parser_for_set)


def _add_positional_profile_arg(parser):
    parser.add_argument(
        action=u"store", dest=main_args.PROFILE_NAME_KEY, help=main_args.PROFILE_HELP_MESSAGE
    )


def _add_authority_arg(parser):
    parser.add_argument(
        u"-s",
        u"--server",
        action=u"store",
        dest=ConfigAccessor.AUTHORITY_KEY,
        help=u"The full scheme, url and port of the Code42 server.",
    )


def _add_username_arg(parser):
    parser.add_argument(
        u"-u",
        u"--username",
        action=u"store",
        dest=ConfigAccessor.USERNAME_KEY,
        help=u"The username of the Code42 API user.",
    )


def _add_disable_ssl_errors_arg(parser):
    parser.add_argument(
        u"--disable-ssl-errors",
        action=u"store_true",
        default=None,
        dest=u"disable_ssl_errors",
        help=u"For development purposes, do not validate the SSL certificates of Code42 servers."
        u"This is not recommended unless it is required.",
    )


def _add_enable_ssl_errors_arg(parser):
    parser.add_argument(
        u"--enable-ssl-errors",
        action=u"store_true",
        default=None,
        dest=u"enable_ssl_errors",
        help=u"Do validate the SSL certificates of Code42 servers.",
    )


def _verify_args_for_set(args):
    if _missing_default_profile(args):
        print_error(u"Must supply a name when setting your profile for the first time.")
        print_set_profile_help()
        exit(1)

    missing_values = not args.c42_username and not args.c42_authority_url
    if missing_values:
        try:
            accessor = get_config_accessor()
            profile = Code42Profile(accessor.get_profile(args.profile_name))
            missing_values = not profile.username and not profile.authority_url
        except Exception:
            missing_values = True

    if missing_values:
        print_error(u"Missing username and authority url.")
        print_set_profile_help()
        exit(1)


def _try_set_authority_url(args, accessor):
    if args.c42_authority_url is not None:
        accessor.set_authority_url(args.c42_authority_url, args.profile_name)


def _try_set_username(args, accessor):
    if args.c42_username is not None:
        accessor.set_username(args.c42_username, args.profile_name)


def _try_set_ignore_ssl_errors(args, accessor):
    if args.disable_ssl_errors is not None and not args.enable_ssl_errors:
        accessor.set_ignore_ssl_errors(True, args.profile_name)

    if args.enable_ssl_errors is not None:
        accessor.set_ignore_ssl_errors(False, args.profile_name)


def _missing_default_profile(args):
    profile_name_arg_is_none = (
        args.profile_name is None or args.profile_name == ConfigAccessor.DEFAULT_VALUE
    )
    return profile_name_arg_is_none and not _default_profile_exist()


def _default_profile_exist():
    try:
        accessor = get_config_accessor()
        profile = Code42Profile(accessor.get_profile())
        return profile.name and profile.name != ConfigAccessor.DEFAULT_VALUE
    except Exception:
        return False


def _prompt_for_allow_password_set(args):
    answer = get_input(u"Would you like to set a password? (y/n): ")
    if answer.lower() == u"y":
        prompt_for_password_reset(args)


def _log_key_save(key):
    if key == ConfigAccessor.DEFAULT_PROFILE_IS_COMPLETE:
        print(u"You have completed setting up your profile!")
    else:
        print(u"'{}' has been successfully updated".format(key))



PROFILE_NAME_KEY = u"profile_name"
PROFILE_HELP_MESSAGE = (
    u"The name of the profile containing your Code42 username and authority host address."
)


def add_arguments_to_parser(parser):
    add_debug_arg(parser)
    add_profile_name_arg(parser)


def add_debug_arg(parser):
    parser.add_argument(
        u"-d",
        u"--debug",
        dest=u"is_debug_mode",
        action=u"store_true",
        help=u"Turn on Debug logging.",
    )


def add_profile_name_arg(parser):
    parser.add_argument(
        u"--profile", action=u"store", dest=PROFILE_NAME_KEY, help=PROFILE_HELP_MESSAGE
    )



from code42cli.securitydata.options import OutputFormat


IS_INCREMENTAL_KEY = u"is_incremental"


def add_arguments_to_parser(parser):
    _add_output_format_arg(parser)
    _add_incremental_arg(parser)


def _add_output_format_arg(parser):
    parser.add_argument(
        u"-f",
        u"--format",
        dest=u"format",
        action=u"store",
        choices=OutputFormat(),
        default=OutputFormat.JSON,
        help=u"The format used for outputting events.",
    )


def _add_incremental_arg(parser):
    parser.add_argument(
        u"-i",
        u"--incremental",
        dest=IS_INCREMENTAL_KEY,
        action=u"store_true",
        help=u"Only get events that were not previously retrieved.",
    )



import code42cli.arguments as main_args
from code42cli.securitydata.arguments import main as securitydata_main_args
from code42cli.securitydata.arguments import search as search_args
from code42cli.securitydata.extraction import extract
from code42cli.securitydata.logger_factory import get_logger_for_stdout


def init(subcommand_parser):
    """Sets up the `print` subcommand.
            Use `-h` after any subcommand for usage.
        Args:
            subcommand_parser: The subparsers group created by the parent parser.
    """
    parser = subcommand_parser.add_parser("print")
    parser.set_defaults(func=print_out)
    search_args.add_arguments_to_parser(parser)
    securitydata_main_args.add_arguments_to_parser(parser)
    main_args.add_arguments_to_parser(parser)


def print_out(args):
    """Activates 'print' command. It gets security events and prints them to stdout."""
    logger = get_logger_for_stdout(args.format)
    extract(logger, args)



from __future__ import print_function

import os
from configparser import ConfigParser

import code42cli.util as util
from code42cli.compat import str


class ConfigAccessor(object):
    DEFAULT_VALUE = u"__DEFAULT__"
    AUTHORITY_KEY = u"c42_authority_url"
    USERNAME_KEY = u"c42_username"
    IGNORE_SSL_ERRORS_KEY = u"ignore-ssl-errors"
    DEFAULT_PROFILE_IS_COMPLETE = u"default_profile_is_complete"
    DEFAULT_PROFILE = u"default_profile"
    _INTERNAL_SECTION = u"Internal"

    def __init__(self, parser):
        self.parser = parser
        self.path = u"{}config.cfg".format(util.get_user_project_path())
        if not os.path.exists(self.path):
            self._create_internal_section()
            self._save()
        else:
            self.parser.read(self.path)

    def get_profile(self, name=None):
        """Returns the profile with the given name.
        If name is None, returns the default profile.
        If the name does not exist or there is no existing profile, it will throw an exception.
        """
        name = name or self._default_profile_name
        if name not in self.parser.sections() or name == self.DEFAULT_VALUE:
            raise Exception(u"Profile does not exist.")
        return self.parser[name]

    def get_all_profiles(self):
        """Returns all the available profiles."""
        profiles = []
        names = self._get_profile_names()
        for name in names:
            profiles.append(self.get_profile(name))
        return profiles

    def create_profile_if_not_exists(self, name):
        """Creates a new profile if one does not already exist for that name."""
        try:
            self.get_profile(name)
        except Exception as ex:
            if name is not None and name != self.DEFAULT_VALUE:
                self._create_profile_section(name)
            else:
                raise ex

    def switch_default_profile(self, new_default_name):
        """Changes what is marked as the default profile in the internal section."""
        if self.get_profile(new_default_name) is None:
            raise Exception(u"Profile does not exist.")
        self._internal[self.DEFAULT_PROFILE] = new_default_name
        self._save()

    def set_authority_url(self, new_value, profile_name=None):
        """Sets 'authority URL' for a given profile.
        Uses the default profile if name is None.
        """
        profile = self.get_profile(profile_name)
        profile[self.AUTHORITY_KEY] = new_value.strip()
        self._save()
        self._try_complete_setup(profile)

    def set_username(self, new_value, profile_name=None):
        """Sets 'username' for a given profile. Uses the default profile if not given a name."""
        profile = self.get_profile(profile_name)
        profile[self.USERNAME_KEY] = new_value.strip()
        self._save()
        self._try_complete_setup(profile)

    def set_ignore_ssl_errors(self, new_value, profile_name=None):
        """Sets 'ignore_ssl_errors' for a given profile.
        Uses the default profile if name is None.
        """
        profile = self.get_profile(profile_name)
        profile[self.IGNORE_SSL_ERRORS_KEY] = str(new_value)
        self._save()

    @property
    def _internal(self):
        """The internal section of the config file."""
        return self.parser[self._INTERNAL_SECTION]

    @property
    def _default_profile_name(self):
        return self._internal[self.DEFAULT_PROFILE]

    def _get_profile_names(self):
        names = list(self.parser.sections())
        names.remove(self._INTERNAL_SECTION)
        return names

    def _create_internal_section(self):
        self.parser.add_section(self._INTERNAL_SECTION)
        self.parser[self._INTERNAL_SECTION] = {}
        self.parser[self._INTERNAL_SECTION][self.DEFAULT_PROFILE_IS_COMPLETE] = str(False)
        self.parser[self._INTERNAL_SECTION][self.DEFAULT_PROFILE] = self.DEFAULT_VALUE

    def _create_profile_section(self, name):
        self.parser.add_section(name)
        self.parser[name] = {}
        self.parser[name][self.AUTHORITY_KEY] = self.DEFAULT_VALUE
        self.parser[name][self.USERNAME_KEY] = self.DEFAULT_VALUE
        self.parser[name][self.IGNORE_SSL_ERRORS_KEY] = str(False)
        default_profile = self._internal.get(self.DEFAULT_PROFILE)
        if default_profile is None or default_profile is self.DEFAULT_VALUE:
            self._internal[self.DEFAULT_PROFILE] = name

    def _save(self):
        util.open_file(self.path, u"w+", lambda f: self.parser.write(f))

    def _try_complete_setup(self, profile):
        if self._internal.getboolean(self.DEFAULT_PROFILE_IS_COMPLETE):
            return

        authority = profile.get(self.AUTHORITY_KEY)
        username = profile.get(self.USERNAME_KEY)

        authority_valid = authority and authority != self.DEFAULT_VALUE
        username_valid = username and username != self.DEFAULT_VALUE

        if not authority_valid or not username_valid:
            return

        self._internal[self.DEFAULT_PROFILE_IS_COMPLETE] = str(True)
        if self._internal[self.DEFAULT_PROFILE] == self.DEFAULT_VALUE:
            self._internal[self.DEFAULT_PROFILE] = profile.name

        self._save()


def get_config_accessor():
    """Create a ConfigAccessor with a ConfigParser as its parser."""
    return ConfigAccessor(ConfigParser())



import platform
from argparse import ArgumentParser

import code42cli.securitydata.main as securitydata
from code42cli.compat import str
from code42cli.profile import profile

# If on Windows, configure console session to handle ANSI escape sequences correctly
# source: https://bugs.python.org/issue29059
if platform.system().lower() == "windows":
    from ctypes import windll, c_int, byref

    stdout_handle = windll.kernel32.GetStdHandle(c_int(-11))
    mode = c_int(0)
    windll.kernel32.GetConsoleMode(c_int(stdout_handle), byref(mode))
    mode = c_int(mode.value | 4)
    windll.kernel32.SetConsoleMode(c_int(stdout_handle), mode)


def main():
    code42_arg_parser = ArgumentParser()
    subcommand_parser = code42_arg_parser.add_subparsers()
    profile.init(subcommand_parser)
    securitydata.init_subcommand(subcommand_parser)
    _run(code42_arg_parser)


def _run(parser):
    try:
        args = parser.parse_args()
        args.func(args)
    except AttributeError as ex:
        if str(ex) == "'Namespace' object has no attribute 'func'":
            parser.print_help()
            return
        raise ex


if __name__ == "__main__":
    main()



from code42cli.securitydata.options import ExposureType


def add_arguments_to_parser(parser):
    _add_advanced_query(parser)
    _add_begin_date_arg(parser)
    _add_end_date_arg(parser)
    _add_exposure_types_arg(parser)
    _add_username_arg(parser)
    _add_actor_arg(parser)
    _add_md5_arg(parser)
    _add_sha256_arg(parser)
    _add_source_arg(parser)
    _add_filename_arg(parser)
    _add_filepath_arg(parser)
    _add_process_owner_arg(parser)
    _add_tab_url_arg(parser)
    _add_include_non_exposure_arg(parser)


class SearchArguments(object):
    ADVANCED_QUERY = u"advanced_query"
    BEGIN_DATE = u"begin_date"
    END_DATE = u"end_date"
    EXPOSURE_TYPES = u"exposure_types"
    C42USERNAME = u"c42usernames"
    ACTOR = u"actors"
    MD5 = u"md5_hashes"
    SHA256 = u"sha256_hashes"
    SOURCE = u"sources"
    FILENAME = u"filenames"
    FILEPATH = u"filepaths"
    PROCESS_OWNER = u"process_owners"
    TAB_URL = u"tab_urls"
    INCLUDE_NON_EXPOSURE_EVENTS = u"include_non_exposure_events"

    def __iter__(self):
        return iter(
            [
                self.ADVANCED_QUERY,
                self.BEGIN_DATE,
                self.END_DATE,
                self.EXPOSURE_TYPES,
                self.C42USERNAME,
                self.ACTOR,
                self.MD5,
                self.SHA256,
                self.SOURCE,
                self.FILENAME,
                self.FILEPATH,
                self.PROCESS_OWNER,
                self.TAB_URL,
                self.INCLUDE_NON_EXPOSURE_EVENTS,
            ]
        )


def _add_advanced_query(parser):
    parser.add_argument(
        u"--advanced-query",
        action=u"store",
        dest=SearchArguments.ADVANCED_QUERY,
        help=u"A raw JSON file event query. "
        u"Useful for when the provided query parameters do not satisfy your requirements."
        u"WARNING: Using advanced queries ignores all other query parameters.",
    )


def _add_begin_date_arg(parser):
    parser.add_argument(
        u"-b",
        u"--begin",
        action=u"store",
        dest=SearchArguments.BEGIN_DATE,
        help=u"The beginning of the date range in which to look for events, "
        u"in YYYY-MM-DD (UTC) or YYYY-MM-DD HH:MM:SS (UTC+24-hr time) format.",
    )


def _add_end_date_arg(parser):
    parser.add_argument(
        u"-e",
        u"--end",
        action=u"store",
        dest=SearchArguments.END_DATE,
        help=u"The end of the date range in which to look for events, "
        u"in YYYY-MM-DD (UTC) or YYYY-MM-DD HH:MM:SS (UTC+24-hr time) format.",
    )


def _add_exposure_types_arg(parser):
    parser.add_argument(
        u"-t",
        u"--types",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.EXPOSURE_TYPES,
        help=u"Limits events to those with given exposure types. "
        u"Available choices={0}".format(list(ExposureType())),
    )


def _add_username_arg(parser):
    parser.add_argument(
        u"--c42username",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.C42USERNAME,
        help=u"Limits events to endpoint events for these users.",
    )


def _add_actor_arg(parser):
    parser.add_argument(
        u"--actor",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.ACTOR,
        help=u"Limits events to only those enacted by these actors.",
    )


def _add_md5_arg(parser):
    parser.add_argument(
        u"--md5",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.MD5,
        help=u"Limits events to file events where the file has one of these MD5 hashes.",
    )


def _add_sha256_arg(parser):
    parser.add_argument(
        u"--sha256",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.SHA256,
        help=u"Limits events to file events where the file has one of these SHA256 hashes.",
    )


def _add_source_arg(parser):
    parser.add_argument(
        u"--source",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.SOURCE,
        help=u"Limits events to only those from one of these sources. Example=Gmail.",
    )


def _add_filename_arg(parser):
    parser.add_argument(
        u"--filename",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.FILENAME,
        help=u"Limits events to file events where the file has one of these names.",
    )


def _add_filepath_arg(parser):
    parser.add_argument(
        u"--filepath",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.FILEPATH,
        help=u"Limits events to file events where the file is located at one of these paths.",
    )


def _add_process_owner_arg(parser):
    parser.add_argument(
        u"--processOwner",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.PROCESS_OWNER,
        help=u"Limits events to exposure events where one of these users "
        u"owns the process behind the exposure.",
    )


def _add_tab_url_arg(parser):
    parser.add_argument(
        u"--tabURL",
        nargs=u"+",
        action=u"store",
        dest=SearchArguments.TAB_URL,
        help=u"Limits events to be exposure events with one of these destination tab URLs.",
    )


def _add_include_non_exposure_arg(parser):
    parser.add_argument(
        u"--include-non-exposure",
        action=u"store_true",
        dest=SearchArguments.INCLUDE_NON_EXPOSURE_EVENTS,
        help=u"Get all events including non-exposure events.",
    )



from __future__ import print_function, with_statement

import sys
from os import path, makedirs
from code42cli.compat import open


def get_input(prompt):
    """Uses correct input function based on Python version."""
    # pylint: disable=undefined-variable
    if sys.version_info >= (3, 0):
        return input(prompt)
    else:
        return raw_input(prompt)


def get_user_project_path(subdir=""):
    """The path on your user dir to /.code42cli/[subdir]."""
    package_name = __name__.split(".")[0]
    home = path.expanduser("~")
    user_project_path = path.join(home, ".{0}".format(package_name), subdir)

    if not path.exists(user_project_path):
        makedirs(user_project_path)

    return user_project_path


def open_file(file_path, mode, action):
    """Wrapper for opening files, useful for testing purposes."""
    with open(file_path, mode, encoding="utf-8") as f:
        action(f)


def print_error(error_text):
    """Prints red text."""
    print("\033[91mERROR: {}\033[0m".format(error_text))


def print_bold(bold_text):
    print("\033[1m{}\033[0m".format(bold_text))


def is_interactive():
    return sys.stdin.isatty()


def print_no_existing_profile_message():
    print_error(u"No existing profile.")
    print_set_profile_help()


def print_set_profile_help():
    print(u"")
    print(u"To add a profile, use: ")
    print_bold(u"\tcode42 profile set --profile <profile-name> -s <authority-URL> -u <username>")
    print(u"")


def get_url_parts(url_str):
    parts = url_str.split(u":")
    port = None
    if len(parts) > 1 and parts[1] != u"":
        port = int(parts[1])
    return parts[0], port



from code42cli.securitydata.subcommands import clear_checkpoint, print_out, write_to
from code42cli.securitydata.subcommands import send_to


def init_subcommand(subcommand_parser):
    securitydata_arg_parser = subcommand_parser.add_parser("securitydata")
    securitydata_subparser = securitydata_arg_parser.add_subparsers()
    send_to.init(securitydata_subparser)
    write_to.init(securitydata_subparser)
    print_out.init(securitydata_subparser)
    clear_checkpoint.init(securitydata_subparser)



from datetime import datetime, timedelta

from c42eventextractor.common import convert_datetime_to_timestamp
from py42.sdk.file_event_query.event_query import EventTimestamp

_MAX_LOOK_BACK_DAYS = 90
_FORMAT_VALUE_ERROR_MESSAGE = u"input must be a date in YYYY-MM-DD or YYYY-MM-DD HH:MM:SS format."


def create_event_timestamp_filter(begin_date=None, end_date=None):
    """Creates a `py42.sdk.file_event_query.event_query.EventTimestamp` filter using the given dates.
        Returns None if not given a begin_date or an end_date.

        Args:
            begin_date: The begin date for the range.
            end_date: The end date for the range.
    """
    end_date = _get_end_date_with_eod_time_if_needed(end_date)
    if begin_date and end_date:
        return _create_in_range_filter(begin_date, end_date)
    elif begin_date and not end_date:
        return _create_on_or_after_filter(begin_date)
    elif end_date and not begin_date:
        return _create_on_or_before_filter(end_date)


def _create_in_range_filter(begin_date, end_date):
    min_timestamp = _parse_min_timestamp(begin_date)
    max_timestamp = _parse_timestamp(end_date)
    _verify_timestamp_order(min_timestamp, max_timestamp)
    return EventTimestamp.in_range(min_timestamp, max_timestamp)


def _create_on_or_after_filter(begin_date):
    min_timestamp = _parse_min_timestamp(begin_date)
    return EventTimestamp.on_or_after(min_timestamp)


def _create_on_or_before_filter(end_date):
    max_timestamp = _parse_timestamp(end_date)
    return EventTimestamp.on_or_before(max_timestamp)


def _get_end_date_with_eod_time_if_needed(end_date):
    if end_date and len(end_date) == 1:
        return end_date[0], "23:59:59"
    return end_date


def _parse_min_timestamp(begin_date_str):
    min_timestamp = _parse_timestamp(begin_date_str)
    boundary_date = datetime.utcnow() - timedelta(days=_MAX_LOOK_BACK_DAYS)
    boundary = convert_datetime_to_timestamp(boundary_date)
    if min_timestamp and min_timestamp < boundary:
        raise ValueError(u"'Begin date' must be within 90 days.")
    return min_timestamp


def _verify_timestamp_order(min_timestamp, max_timestamp):
    if min_timestamp is None or max_timestamp is None:
        return
    if min_timestamp >= max_timestamp:
        raise ValueError(u"Begin date cannot be after end date")


def _parse_timestamp(date_and_time):
    try:
        date_str = _join_date_and_time(date_and_time)
        date_format = u"%Y-%m-%d" if len(date_and_time) == 1 else u"%Y-%m-%d %H:%M:%S"
        time = datetime.strptime(date_str, date_format)
    except ValueError:
        raise ValueError(_FORMAT_VALUE_ERROR_MESSAGE)
    return convert_datetime_to_timestamp(time)


def _join_date_and_time(date_and_time):
    if not date_and_time:
        return None
    date_str = date_and_time[0]
    if len(date_and_time) == 1:
        return date_str
    if len(date_and_time) == 2:
        date_str = "{0} {1}".format(date_str, date_and_time[1])
    else:
        raise ValueError(_FORMAT_VALUE_ERROR_MESSAGE)
    return date_str



import logging
import sys
from logging.handlers import RotatingFileHandler
from threading import Lock

from c42eventextractor.logging.formatters import (
    FileEventDictToJSONFormatter,
    FileEventDictToCEFFormatter,
    FileEventDictToRawJSONFormatter,
)
from c42eventextractor.logging.handlers import NoPrioritySysLogHandlerWrapper

from code42cli.compat import str
from code42cli.securitydata.options import OutputFormat
from code42cli.util import get_user_project_path, print_error, get_url_parts

_logger_deps_lock = Lock()


def get_logger_for_stdout(output_format):
    """Gets the stdout logger for the given format.

        Args:
            output_format: CEF, JSON, or RAW_JSON. Each type results in a different logger instance.
    """
    logger = logging.getLogger(u"code42_stdout_{0}".format(output_format.lower()))
    if _logger_has_handlers(logger):
        return logger

    with _logger_deps_lock:
        if not _logger_has_handlers(logger):
            handler = logging.StreamHandler(sys.stdout)
            return _init_logger(logger, handler, output_format)
    return logger


def get_logger_for_file(filename, output_format):
    """Gets the logger that logs to a file for the given format.

        Args:
            filename: The name of the file to write logs to.
            output_format: CEF, JSON, or RAW_JSON. Each type results in a different logger instance.
    """
    logger = logging.getLogger(u"code42_file_{0}".format(output_format.lower()))
    if _logger_has_handlers(logger):
        return logger

    with _logger_deps_lock:
        if not _logger_has_handlers(logger):
            handler = logging.FileHandler(filename, delay=True, encoding="utf-8")
            return _init_logger(logger, handler, output_format)
    return logger


def get_logger_for_server(hostname, protocol, output_format):
    """Gets the logger that sends logs to a server for the given format.

        Args:
            hostname: The hostname of the server. It may include the port.
            protocol: The transfer protocol for sending logs.
            output_format: CEF, JSON, or RAW_JSON. Each type results in a different logger instance.
    """
    logger = logging.getLogger(u"code42_syslog_{0}".format(output_format.lower()))
    if _logger_has_handlers(logger):
        return logger

    with _logger_deps_lock:
        if not _logger_has_handlers(logger):
            url_parts = get_url_parts(hostname)
            port = url_parts[1] or 514
            handler = NoPrioritySysLogHandlerWrapper(
                url_parts[0], port=port, protocol=protocol
            ).handler
            return _init_logger(logger, handler, output_format)
    return logger


def get_error_logger():
    """Gets the logger where exceptions are logged."""
    log_path = get_user_project_path(u"log")
    log_path = u"{0}/code42_errors.log".format(log_path)
    logger = logging.getLogger(u"code42_error_logger")
    if _logger_has_handlers(logger):
        return logger

    with _logger_deps_lock:
        if not _logger_has_handlers(logger):
            formatter = logging.Formatter(u"%(asctime)s %(message)s")
            handler = RotatingFileHandler(log_path, maxBytes=250000000, encoding="utf-8")
            return _apply_logger_dependencies(logger, handler, formatter)
    return logger


def _logger_has_handlers(logger):
    return len(logger.handlers)


def _init_logger(logger, handler, output_format):
    formatter = _get_formatter(output_format)
    logger.setLevel(logging.INFO)
    return _apply_logger_dependencies(logger, handler, formatter)


def _apply_logger_dependencies(logger, handler, formatter):
    try:
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    except Exception as ex:
        print_error(str(ex))
        exit(1)
    return logger


def _get_formatter(output_format):
    if output_format == OutputFormat.JSON:
        return FileEventDictToJSONFormatter()
    elif output_format == OutputFormat.CEF:
        return FileEventDictToCEFFormatter()
    else:
        return FileEventDictToRawJSONFormatter()



from __future__ import with_statement

import sqlite3

from code42cli.util import get_user_project_path

_INSERTION_TIMESTAMP_FIELD_NAME = u"insertionTimestamp"


class BaseCursorStore(object):
    _PRIMARY_KEY_COLUMN_NAME = u"cursor_id"

    def __init__(self, db_table_name, db_file_path=None):
        self._table_name = db_table_name
        if db_file_path is None:
            db_path = get_user_project_path(u"db")
            db_file_path = u"{0}/{1}.db".format(db_path, self._table_name)

        self._connection = sqlite3.connect(db_file_path)

    def _get(self, columns, primary_key):
        query = u"SELECT {0} FROM {1} WHERE {2}=?"
        query = query.format(columns, self._table_name, self._PRIMARY_KEY_COLUMN_NAME)
        with self._connection as conn:
            cursor = conn.cursor()
            cursor.execute(query, (primary_key,))
            return cursor.fetchall()

    def _set(self, column_name, new_value, primary_key):
        query = u"UPDATE {0} SET {1}=? WHERE {2}=?".format(
            self._table_name, column_name, self._PRIMARY_KEY_COLUMN_NAME
        )
        with self._connection as conn:
            conn.execute(query, (new_value, primary_key))

    def _row_exists(self, primary_key):
        query = u"SELECT * FROM {0} WHERE {1}=?"
        query = query.format(self._table_name, self._PRIMARY_KEY_COLUMN_NAME)
        with self._connection as conn:
            cursor = conn.cursor()
            cursor.execute(query, (primary_key,))
            query_result = cursor.fetchone()
            if not query_result:
                return False
            return True

    def _drop_table(self):
        drop_query = u"DROP TABLE {0}".format(self._table_name)
        with self._connection as conn:
            conn.execute(drop_query)

    def _is_empty(self):
        table_count_query = u"""
            SELECT COUNT(name)
            FROM sqlite_master
            WHERE type='table' AND name=?
        """
        with self._connection as conn:
            cursor = conn.cursor()
            cursor.execute(table_count_query, (self._table_name,))
            query_result = cursor.fetchone()
            if query_result:
                return int(query_result[0]) <= 0


class FileEventCursorStore(BaseCursorStore):
    def __init__(self, profile_name, db_file_path=None):
        self._primary_key = profile_name
        super(FileEventCursorStore, self).__init__(u"file_event_checkpoints", db_file_path)
        if self._is_empty():
            self._init_table()
        if not self._row_exists(self._primary_key):
            self._insert_new_row()

    def get_stored_insertion_timestamp(self):
        """Gets the last stored insertion timestamp."""
        rows = self._get(_INSERTION_TIMESTAMP_FIELD_NAME, self._primary_key)
        if rows and rows[0]:
            return rows[0][0]

    def replace_stored_insertion_timestamp(self, new_insertion_timestamp):
        """Replaces the last stored insertion timestamp with the given one."""
        self._set(
            column_name=_INSERTION_TIMESTAMP_FIELD_NAME,
            new_value=new_insertion_timestamp,
            primary_key=self._primary_key,
        )

    def _init_table(self):
        columns = u"{0}, {1}".format(self._PRIMARY_KEY_COLUMN_NAME, _INSERTION_TIMESTAMP_FIELD_NAME)
        create_table_query = u"CREATE TABLE {0} ({1})".format(self._table_name, columns)
        with self._connection as conn:
            conn.execute(create_table_query)

    def _insert_new_row(self):
        insert_query = u"INSERT INTO {0} VALUES(?, null)".format(self._table_name)
        with self._connection as conn:
            conn.execute(insert_query, (self._primary_key,))
